"""empty message

Revision ID: 5ade3d942429
Revises:
Create Date: 2015-04-17 10:23:56.546052

"""

# revision identifiers, used by Alembic.
revision = '5ade3d942429'
down_revision = None
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils


def upgrade():
    TYPES = (
        ('repo.push', 'Repository Push'),
    )

    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('webhooks',
    sa.Column('uuid', sqlalchemy_utils.types.uuid.UUIDType(binary=False), nullable=False),
    sa.Column('owner', sa.UnicodeText(), nullable=True),
    sa.Column('url', sqlalchemy_utils.types.url.URLType(), nullable=False),
    sa.Column('event_type', sqlalchemy_utils.types.choice.ChoiceType(TYPES), nullable=False),
    sa.Column('active', sa.Boolean(), nullable=False),
    sa.PrimaryKeyConstraint('uuid'),
    sa.UniqueConstraint('owner', 'uuid')
    )

def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('webhooks')
    ### end Alembic commands ###



import os
import json

from avro.io import validate

from colander import MappingSchema, SchemaNode, String, Invalid

from unicore.distribute.utils import get_config, get_repository, get_schema


def validate_schema(request):
    config = get_config(request)
    storage_path = config.get('repo.storage_path')
    repo = get_repository(
        os.path.join(storage_path, request.matchdict['name']))
    uuid = request.matchdict['uuid']
    content_type = request.matchdict['content_type']
    schema = get_schema(repo, content_type)
    data = json.loads(request.body)

    if not validate(schema, data):
        request.errors.status = 400
        request.errors.add(
            'body',
            'schema',
            'Data does not match the schema for %s' % (content_type,)
        )
    elif uuid is not None and data['uuid'] != uuid:
        request.errors.status = 400
        request.errors.add(
            'body',
            'uuid',
            'Payload UUID does not match URL UUID.'
        )
    else:
        request.schema = schema.to_json()
        request.schema_data = data


def repo_url_type_schema_validator(node, value):
    valid_prefixes = ['git://', 'http://', 'https://', 'ssh://']
    if not any([value.startswith(prefix) for prefix in valid_prefixes]):
        raise Invalid(node, '%r is not a valid repo_url' % (value,))


class CreateRepoColanderSchema(MappingSchema):
    repo_url = SchemaNode(
        String(), location='body', validator=repo_url_type_schema_validator)



import os
import json

from elasticgit.commands.avro import serialize
from elasticgit.tests.base import TestPerson, ModelBaseTest
from elasticgit.utils import fqcn

from colander import Invalid

from cornice.errors import Errors

from unicore.distribute.api.validators import (
    validate_schema, repo_url_type_schema_validator, CreateRepoColanderSchema)

from pyramid import testing


class TestValidators(ModelBaseTest):

    def setUp(self):
        self.workspace = self.mk_workspace()

        schema_string = serialize(TestPerson)
        schema = json.loads(schema_string)
        self.workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')

        self.person = TestPerson({'name': 'Foo', 'age': 1})
        self.request = testing.DummyRequest()
        self.request.errors = Errors()
        self.config = testing.setUp(
            settings={
                'repo.storage_path': self.WORKING_DIR,
            }, request=self.request)

    def test_validate_schema_valid(self):
        self.request.matchdict = {
            'name': os.path.basename(self.workspace.working_dir),
            'content_type': fqcn(TestPerson),
            'uuid': self.person.uuid,
        }
        self.request.body = json.dumps(dict(self.person))
        validate_schema(self.request)
        self.assertEqual(self.request.errors, [])

    def test_validate_schema_invalid(self):
        self.request.matchdict = {
            'name': os.path.basename(self.workspace.working_dir),
            'content_type': fqcn(TestPerson),
            'uuid': self.person.uuid,
        }
        self.request.body = json.dumps({'age': 'twenty two'})
        validate_schema(self.request)
        self.assertEqual(
            self.request.errors, [{
                'location': 'body',
                'name': 'schema',
                'description': (
                    'Data does not match the schema for '
                    'elasticgit.tests.base.TestPerson')
            }])

    def test_validate_schema_invalid_uuid(self):
        self.request.matchdict = {
            'name': os.path.basename(self.workspace.working_dir),
            'content_type': fqcn(TestPerson),
            'uuid': 'foo',
        }
        self.request.body = json.dumps(dict(self.person))
        validate_schema(self.request)
        self.assertEqual(
            self.request.errors, [{
                'location': 'body',
                'name': 'uuid',
                'description': 'Payload UUID does not match URL UUID.'}])

    def test_repo_url_type_schema_validator(self):
        self.assertRaises(
            Invalid, repo_url_type_schema_validator, None, 'file://foo')
        self.assertRaises(
            Invalid, repo_url_type_schema_validator, None, '//foo')
        self.assertRaises(
            Invalid, repo_url_type_schema_validator, None, '..//foo')
        self.assertEqual(
            None, repo_url_type_schema_validator(None, 'http://foo/bar.git'))
        self.assertEqual(
            None, repo_url_type_schema_validator(None, 'https://foo/bar.git'))
        self.assertEqual(
            None, repo_url_type_schema_validator(None, 'ssh://foo/bar.git'))
        self.assertEqual(
            None, repo_url_type_schema_validator(None, 'git://foo/bar.git'))

    def test_create_repo_colander_schema(self):
        schema = CreateRepoColanderSchema()
        valid = schema.deserialize({'repo_url': 'http://example.org/foo.git'})
        self.assertEqual(valid, {'repo_url': 'http://example.org/foo.git'})
        self.assertRaises(Invalid, schema.deserialize, {'repo_url': 'foo'})



import os
import argparse

from elasticgit.storage import StorageManager

from pyramid.paster import bootstrap
from pyramid.request import Request

from unicore.distribute.utils import get_repositories
from unicore.webhooks.events import WebhookEvent


class PollRepositories(object):

    # This gets set at run time or is overriden for tests
    notify = None

    def run(self, repo_dir, ini_file, base_url):
        request = Request.blank('/', base_url=base_url)
        env = bootstrap(ini_file, request=request)
        self.notify = env['registry'].notify
        for repo in get_repositories(repo_dir):
            self.pull_repo(env, repo)

        # http://docs.pylonsproject.org/projects/pyramid/en/1.3-branch/narr/commandline.html#cleanup # noqa
        env['closer']()

    def pull_repo(self, env, repo):
        sm = StorageManager(repo)
        branch = repo.active_branch
        tracking_branch = branch.tracking_branch()
        if tracking_branch:
            remote_name = tracking_branch.remote_name
        else:
            remote_name = repo.remotes[0].name
        original_commit = branch.commit
        sm.pull(branch_name=branch.name,
                remote_name=remote_name)
        last_commit = branch.commit
        if original_commit.hexsha != last_commit.hexsha:
            name = os.path.basename(repo.working_dir)
            request = env['request']
            self.notify(
                WebhookEvent(
                    owner=None,
                    event_type='repo.push',
                    payload={
                        'repo': name,
                        'url': request.route_url(
                            'repositoryresource', name=name)
                    }))


def get_parser():  # pragma: no cover
    parser = argparse.ArgumentParser(
        description="unicore.distribute command line tools.")
    subparser = parser.add_subparsers(help='Commands')
    command = subparser.add_parser(
        'poll-repositories', help='poll repositories')
    command.add_argument(
        '-d', '--repo-dir',
        dest='repo_dir',
        help='The directory with repositories.',
        default='./repos')
    command.add_argument(
        '-i', '--ini-file',
        dest='ini_file',
        help='The project\'s ini file.',
        default='development.ini')
    command.add_argument(
        '-u', '--base-url',
        dest='base_url',
        help='This server\'s public URL (for webhooks)',
        default='http://localhost/')
    command.set_defaults(dispatcher=PollRepositories)

    return parser


def run(parser):  # pragma: no cover
    args = parser.parse_args()
    data = vars(args)
    dispatcher_class = data.pop('dispatcher')
    dispatcher = dispatcher_class()
    dispatcher.run(**data)


def main():  # pragma: no cover
    parser = get_parser()
    run(parser)



from unittest import TestCase

from mock import Mock, patch

from pyramid import testing
from pyramid.request import Request
from webtest import TestApp

from requests.models import Response

from unicore.distribute.api.proxy import ProxyView, Proxy
from unicore.distribute.api import main


class TestProxy(TestCase):

    def mk_proxy_view(self,
                      url='http://example.org',
                      method='GET',
                      response_content='',
                      content_type='application/json',
                      encoding='utf-8'):
        request = Request.blank('/', base_url=url, method=method)
        request.matchdict = {
            'parts': '',
        }

        proxy_view = ProxyView(request, url)
        response = Response()
        response.headers['Content-Type'] = content_type
        response._content = response_content
        response.encoding = encoding
        mock = Mock()
        mock.return_value = response
        proxy_view.mk_request = mock
        return proxy_view

    def mk_proxy_request(self, method, response_content=''):
        proxy_view = self.mk_proxy_view(
            'http://example.org', response_content=response_content)
        return getattr(proxy_view, 'do_%s' % (method,))()

    def test_proxy_view(self):
        proxy_view = self.mk_proxy_view('http://example.org')
        self.assertTrue(isinstance(proxy_view, ProxyView))

    def test_do_POST(self):
        proxy_view = self.mk_proxy_view(method='POST')
        proxy_view.do_POST()
        proxy_view.mk_request.assert_called_with(
            'POST', 'http://example.org', data='')

    def test_do_GET(self):
        proxy_view = self.mk_proxy_view(method='GET')
        proxy_view.do_GET()
        proxy_view.mk_request.assert_called_with(
            'GET', 'http://example.org', data='')

    def test_do_DELETE(self):
        proxy_view = self.mk_proxy_view(method='DELETE')
        proxy_view.do_DELETE()
        proxy_view.mk_request.assert_called_with(
            'DELETE', 'http://example.org', data='')

    def test_do_PUT(self):
        proxy_view = self.mk_proxy_view(method='PUT')
        proxy_view.do_PUT()
        proxy_view.mk_request.assert_called_with(
            'PUT', 'http://example.org', data='')

    def test_do_HEAD(self):
        proxy_view = self.mk_proxy_view(method='HEAD')
        proxy_view.do_HEAD()
        proxy_view.mk_request.assert_called_with(
            'HEAD', 'http://example.org', data='')

    def test_unsupported_methods(self):
        request = Request.blank(
            '/', base_url='http://www.example.org', method='PATCH')
        request.matchdict = {'parts': 'foo'}

        proxy = Proxy('http://example.org')
        resp = proxy(request)
        self.assertEqual(resp.status_code, 404)

    @patch.object(ProxyView, 'mk_request')
    def test_proxy_setup(self, mocked_request):
        response = Response()
        response.headers['Content-Type'] = 'application/text'
        response._content = 'hello world'
        response.encoding = 'utf-8'
        mocked_request.return_value = response
        settings = {'proxy.enabled': 'true'}
        testing.setUp(settings=settings)
        app = TestApp(main({}, **settings))
        response = app.get('/esapi/')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers['content-type'], 'application/text')
        self.assertEqual(response.body, 'hello world')



import json

from git import Repo
import os
from elasticgit.commands.avro import serialize
from elasticgit.tests.base import ModelBaseTest, TestPerson
from pyramid import testing
from pyramid.exceptions import NotFound
from unicore.distribute.api.repo_status import (
    RepositoryStatusResource, RepositoryDiffResource, RepositoryPullResource,
    RepositoryCloneResource)
from unicore.distribute.utils import (
    format_repo_status, get_repository_diff, pull_repository_files,
    clone_repository)


class TestRepositoryStatusResource(ModelBaseTest):
    def setUp(self):
        self.workspace = self.mk_workspace()
        self.add_schema(self.workspace, TestPerson)
        self.config = testing.setUp(settings={
            'repo.storage_path': self.WORKING_DIR,
        })

    def add_schema(self, workspace, model_class):
        schema_string = serialize(model_class)
        schema = json.loads(schema_string)
        workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')

    def test_get(self):
        request = testing.DummyRequest({})
        repo_name = os.path.basename(self.workspace.working_dir)
        request.matchdict = {
            'name': repo_name,
        }
        resource = RepositoryStatusResource(request)
        repo_json = resource.get()
        self.assertEqual(repo_json, format_repo_status(self.workspace.repo))

    def test_get_404(self):
        request = testing.DummyRequest({})
        request.matchdict = {
            'name': 'does-not-exist',
        }
        resource = RepositoryStatusResource(request)
        self.assertRaises(NotFound, resource.get)


class TestRepositoryDiffResource(ModelBaseTest):
    initial_commit = ""

    def setUp(self):
        self.workspace = self.mk_workspace()
        self.add_schema(self.workspace, TestPerson)
        self.config = testing.setUp(settings={
            'repo.storage_path': self.WORKING_DIR,
        })
        self.initial_commit = self.create_commit("initial commit")
        person1 = TestPerson({'age': 12, 'name': 'Foo'})
        person2 = TestPerson({'age': 34, 'name': 'Bar'})
        self.workspace.save(person1, "saving person 1")
        self.workspace.save(person2, "saving person 2")
        self.create_commit("second commit")

    def add_schema(self, workspace, model_class):
        schema_string = serialize(model_class)
        schema = json.loads(schema_string)
        workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')

    def create_commit(self, message):
        repo = Repo(self.workspace.working_dir)
        repo.commit(repo.index.commit(message))
        return repo.commit().hexsha

    def test_get(self):
        request = testing.DummyRequest({})
        repo_name = os.path.basename(self.workspace.working_dir)
        request.matchdict = {
            'name': repo_name,
            'commit_id': self.initial_commit
        }
        resource = RepositoryDiffResource(request)
        repo_json = resource.get()
        self.assertTrue(len(repo_json))
        self.assertEqual(repo_json, get_repository_diff(
            self.workspace.repo, self.initial_commit))

    def test_get_404(self):
        request = testing.DummyRequest({})
        repo_name = os.path.basename(self.workspace.working_dir)
        request.matchdict = {
            'name': repo_name,
            'commit_id': 1234567
        }
        resource = RepositoryDiffResource(request)
        self.assertRaises(NotFound, resource.get)


class TestRepositoryPullResource(ModelBaseTest):
    initial_commit = ""

    def setUp(self):
        self.workspace = self.mk_workspace()
        self.add_schema(self.workspace, TestPerson)
        self.config = testing.setUp(settings={
            'repo.storage_path': self.WORKING_DIR,
        })
        self.initial_commit = self.create_commit("initial commit")
        person1 = TestPerson({'age': 12, 'name': 'Foo'})
        person2 = TestPerson({'age': 34, 'name': 'Bar'})
        self.workspace.save(person1, "saving person 1")
        self.workspace.save(person2, "saving person 2")
        self.create_commit("second commit")

    def add_schema(self, workspace, model_class):
        schema_string = serialize(model_class)
        schema = json.loads(schema_string)
        workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')

    def create_commit(self, message):
        repo = Repo(self.workspace.working_dir)
        repo.index.commit(message)
        return repo.commit().hexsha

    def test_get(self):
        request = testing.DummyRequest({})
        repo_name = os.path.basename(self.workspace.working_dir)
        request.matchdict = {
            'name': repo_name,
            'commit_id': self.initial_commit
        }
        resource = RepositoryPullResource(request)
        repo_json = resource.get()
        self.assertTrue(len(repo_json))
        self.assertEqual(repo_json, pull_repository_files(
            self.workspace.repo, self.initial_commit))

    def test_get_404(self):
        request = testing.DummyRequest({})
        repo_name = os.path.basename(self.workspace.working_dir)
        request.matchdict = {
            'name': repo_name,
            'commit_id': '12345678'
        }
        resource = RepositoryPullResource(request)
        self.assertRaises(NotFound, resource.get)


class TestRepositoryCloneResource(ModelBaseTest):
    def setUp(self):
        self.workspace = self.mk_workspace()
        self.add_schema(self.workspace, TestPerson)
        self.config = testing.setUp(settings={
            'repo.storage_path': self.WORKING_DIR,
        })
        person1 = TestPerson({'age': 12, 'name': 'Foo'})
        person2 = TestPerson({'age': 34, 'name': 'Bar'})
        self.create_commit("initial commit")
        self.workspace.save(person1, "saving person 1")
        self.workspace.save(person2, "saving person 2")
        self.create_commit("second commit")

    def add_schema(self, workspace, model_class):
        schema_string = serialize(model_class)
        schema = json.loads(schema_string)
        workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')

    def create_commit(self, message):
        repo = Repo(self.workspace.working_dir)
        repo.index.commit(message)
        return repo.commit().hexsha

    def test_get(self):
        request = testing.DummyRequest({})
        repo_name = os.path.basename(self.workspace.working_dir)
        request.matchdict = {
            'name': repo_name,
        }
        resource = RepositoryCloneResource(request)
        repo_json = resource.get()
        self.assertTrue(len(repo_json))
        self.assertEqual(repo_json, clone_repository(self.workspace.repo))

    def test_get_404(self):
        request = testing.DummyRequest({})
        request.matchdict = {
            'name': 'invalid-repo'
        }
        resource = RepositoryCloneResource(request)
        self.assertRaises(NotFound, resource.get)



from pyramid.response import Response
from pyramid.httpexceptions import HTTPNotFound

import requests


class Proxy(object):

    def __init__(self, upstream_url):
        self.upstream_url = upstream_url

    def __call__(self, request):
        view = ProxyView(request, self.upstream_url)
        handler = getattr(view, 'do_%s' % (request.method,), HTTPNotFound)
        return handler()


class ProxyView(object):

    def __init__(self, request, upstream_url):
        self.request = request
        self.upstream_url = upstream_url

    def url(self):
        return '%s%s' % (self.upstream_url, self.request.matchdict['parts'])

    def mk_request(self, *args, **kwargs):  # for mocking
        return requests.request(*args, **kwargs)

    def mk_response(self, response):
        return Response(body=response.text, status=response.status_code,
                        headerlist=response.headers.items(),
                        content_type=response.headers['Content-Type'],
                        charset=response.encoding)

    def do_request(self):
        return self.mk_response(
            self.mk_request(
                self.request.method, self.url(), data=self.request.body))

    def do_POST(self):
        return self.do_request()

    def do_DELETE(self):
        return self.do_request()

    def do_PUT(self):
        return self.do_request()

    def do_GET(self):
        return self.do_request()

    def do_HEAD(self):
        return self.do_request()



import os.path

from ConfigParser import ConfigParser
from StringIO import StringIO

from pyramid.paster import bootstrap
from pyramid.request import Request

from elasticgit.tests.base import ToolBaseTest

from unicore.distribute.scripts import PollRepositories

from elasticgit.tests.base import TestPerson

from mock import Mock, patch


class TestScripts(ToolBaseTest):

    def setUp(self):
        self.workspace = self.mk_workspace()
        self.remote_workspace = self.mk_workspace(
            name='%s_remote' % (self.id().lower(),),
            index_prefix='%s_remote' % (self.workspace.index_prefix,))

        person1 = TestPerson({
            'age': 1,
            'name': 'Name'
        })

        self.remote_workspace.save(person1, 'Saving person1.')
        self.workspace.repo.create_remote(
            'origin', self.remote_workspace.working_dir)
        self.workspace.fast_forward()

    def mk_inifile(self, data):
        cp = ConfigParser()
        for section, key_data in data.items():
            cp.add_section(section)
            for key, value in key_data.items():
                cp.set(section, key, value)

        sio = StringIO()
        cp.write(sio)

        tmp_ini_data = sio.getvalue()
        tmp_ini_file = self.mk_tempfile(tmp_ini_data)
        return tmp_ini_file

    @patch.object(PollRepositories, 'pull_repo')
    def test_run(self, mocked_pull_repo):
        pr = PollRepositories()
        pr.run(self.WORKING_DIR, self.mk_inifile({
            'app:main': {
                'use': 'egg:unicore.distribute',
                'repo.storage_path': self.WORKING_DIR,
            }
        }), 'http://www.example.org')
        mocked_pull_repo.assert_called()
        (ws_call, remote_ws_call) = mocked_pull_repo.call_args_list
        (args, kwargs) = ws_call
        (env, repo) = args
        self.assertEqual(repo, self.workspace.repo)

    def test_pull_repo(self):

        from pyramid.events import subscriber
        from unicore.webhooks.events import WebhookEvent

        mock = Mock()
        subscriber(WebhookEvent)(mock)

        request = Request.blank('/', base_url='http://example.org')

        tmp_ini_file = self.mk_inifile({
            'app:main': {
                'use': 'egg:unicore.distribute',
                'repo.storage_path': self.WORKING_DIR
            }
        })
        env = bootstrap(tmp_ini_file, request=request)

        mock = Mock()
        poll_repos = PollRepositories()
        poll_repos.notify = mock
        poll_repos.pull_repo(env, self.workspace.repo)

        mock.assert_not_called()

        self.remote_workspace.save(
            TestPerson({'age': 2, 'name': 'Foo'}), 'Saving person2')

        poll_repos.pull_repo(env, self.workspace.repo)
        mock.assert_called()
        (call,) = mock.call_args_list
        (args, kwargs) = call
        (event,) = args
        name = os.path.basename(self.workspace.working_dir)
        url = env['request'].route_url('repositoryresource', name=name)
        self.assertTrue('http://example.org' in url)
        self.assertEqual(event.owner, None)
        self.assertEqual(event.event_type, 'repo.push')
        self.assertEqual(event.payload, {
            'repo': name,
            'url': url,
        })



import os
from cornice.resource import resource, view
from unicore.distribute.utils import (
    get_config, format_repo_status, get_repository, get_repository_diff,
    pull_repository_files, clone_repository)


@resource(path='/repos/{name}/status.json')
class RepositoryStatusResource(object):
    def __init__(self, request):
        self.request = request
        self.config = get_config(request)

    @view(renderer='json')
    def get(self):
        name = self.request.matchdict['name']
        storage_path = self.config.get('repo.storage_path')
        return format_repo_status(get_repository(
            os.path.join(storage_path, name)))


@resource(path='/repos/{name}/diff/{commit_id}.json')
class RepositoryDiffResource(object):
    def __init__(self, request):
        self.request = request
        self.config = get_config(request)

    @view(renderer='json')
    def get(self):
        name = self.request.matchdict['name']
        commit_id = self.request.matchdict['commit_id']
        storage_path = self.config.get('repo.storage_path')
        return get_repository_diff(get_repository(
            os.path.join(storage_path, name)), commit_id)


@resource(path='/repos/{name}/pull/{commit_id}.json')
class RepositoryPullResource(object):
    def __init__(self, request):
        self.request = request
        self.config = get_config(request)

    @view(renderer='json')
    def get(self):
        name = self.request.matchdict['name']
        commit_id = self.request.matchdict['commit_id']
        storage_path = self.config.get('repo.storage_path')
        return pull_repository_files(get_repository(
            os.path.join(storage_path, name)), commit_id)


@resource(path='/repos/{name}/clone.json')
class RepositoryCloneResource(object):
    def __init__(self, request):
        self.request = request
        self.config = get_config(request)

    @view(renderer='json')
    def get(self):
        name = self.request.matchdict['name']
        storage_path = self.config.get('repo.storage_path')
        return clone_repository(
            get_repository(os.path.join(storage_path, name)))



import glob
import json
import os
import re

from datetime import datetime

from ConfigParser import ConfigParser
from pyramid.exceptions import NotFound

from git import Repo
from git.exc import (
    InvalidGitRepositoryError, NoSuchPathError, GitCommandError, BadName)

import avro.schema

from elasticgit.commands.avro import deserialize
from elasticgit.storage import StorageManager


class UCConfigParser(ConfigParser):
    """
    A config parser that understands lists and dictionaries.
    """

    def get_list(self, section, option):
        """
        This allows for loading of Pyramid list style configuration
        options:

        [foo]
        bar =
            baz
            qux
            zap

        ``get_list('foo', 'bar')`` returns ``['baz', 'qux', 'zap']``

        :param str section:
            The section to read.
        :param str option:
            The option to read from the section.
        :returns: list
        """
        value = self.get(section, option)
        return list(filter(None, (x.strip() for x in value.splitlines())))

    def get_dict(self, section, option):
        """
        This allows for loading of Pyramid dictionary style configuration
        options:

        [foo]
        bar =
            baz=qux
            zap=paz

        ``get_dict('foo', 'bar')`` returns ``{'baz': 'qux', 'zap': 'paz'}``

        :param str section:
            The section to read.
        :param str option:
            The option to read from the section.
        :returns: dict
        """
        return dict(re.split('\s*=\s*', value)
                    for value in self.get_list(section, option))


def get_repositories(path):
    """
    Return an array of tuples with the name and path for
    repositories found in a directory.

    :param str path:
        The path to find repositories in
    :returns: tuple
    """
    return [get_repository(os.path.join(path, subdir))
            for subdir in os.listdir(path)
            if os.path.isdir(
            os.path.join(path, subdir, '.git'))]


def get_repository(path):
    """
    Return a repository for whatever's at a path

    :param str path:
        The path to the repository
    :returns: Repo
    """
    try:
        return Repo(path)
    except (NoSuchPathError, InvalidGitRepositoryError):
        raise NotFound('Repository not found.')


def list_schemas(repo):
    """
    Return a list of parsed avro schemas as dictionaries.

    :param Repo repo:
        The git repository.
    :returns: dict
    """
    schema_files = glob.glob(
        os.path.join(repo.working_dir, '_schemas', '*.avsc'))
    schemas = {}
    for schema_file in schema_files:
        with open(schema_file, 'r') as fp:
            schema = json.load(fp)
            schemas['%(namespace)s.%(name)s' % schema] = schema
    return schemas


def get_schema(repo, content_type):
    """
    Return a schema for a content type in a repository.

    :param Repo repo:
        The git repository.
    :returns: dict
    """
    try:
        with open(
                os.path.join(repo.working_dir,
                             '_schemas',
                             '%s.avsc' % (content_type,)), 'r') as fp:
            data = fp.read()
            return avro.schema.parse(data)
    except IOError:  # pragma: no cover
        raise NotFound('Schema does not exist.')


def format_repo(repo):
    """
    Return a dictionary representing the repository

    It returns ``None`` for things we do not support or are not
    relevant.

    :param str repo_name:
        The name of the repository.
    :param git.Repo repo:
        The repository object.
    :param str base_url:
        The base URL for the repository's links.
    :returns: dict

    """
    commit = repo.commit()
    return {
        'name': os.path.basename(repo.working_dir),
        'branch': repo.active_branch.name,
        'commit': commit.hexsha,
        'timestamp': datetime.fromtimestamp(
            commit.committed_date).isoformat(),
        'author': '%s <%s>' % (commit.author.name, commit.author.email),
        'schemas': list_schemas(repo)
    }


def format_diff_A(diff):
    return {
        'type': 'A',
        'path': diff.b_blob.path,
    }


def format_diff_D(diff):
    return {
        'type': 'D',
        'path': diff.a_blob.path,
    }


def format_diff_R(diff):
    return {
        'type': 'R',
        'rename_from': diff.rename_from,
        'rename_to': diff.rename_to,
    }


def format_diff_M(diff):
    return {
        'type': 'M',
        'path': diff.a_blob.path,
    }


def format_diffindex(diff_index):
    """
    Return a JSON formattable representation of a DiffIndex.

    Returns a generator that returns dictionaries representing the changes.

    .. code::
        [
            {
                'type': 'A',
                'path': 'path/to/added/file.txt',
            },
            {
                'type': 'D',
                'path': 'path/to/deleted/file.txt',
            },
            {
                'type': 'M',
                'path': 'path/to/modified/file.txt',
            },
            {
                'type': 'R',
                'rename_from': 'original/path/to/file.txt',
                'rename_to': 'new/path/to/file.txt',
            },
        ]

    :returns: generator

    """
    for diff in diff_index:
        if diff.new_file:
            yield format_diff_A(diff)
        elif diff.deleted_file:
            yield format_diff_D(diff)
        elif diff.renamed:
            yield format_diff_R(diff)
        elif diff.a_blob and diff.b_blob and diff.a_blob != diff.b_blob:
            yield format_diff_M(diff)


def format_content_type(repo, content_type):
    """
    Return a list of all content objects for a given content type
    in a repository.

    :param Repo repo:
        The git repository.
    :param str content_type:
        The content type to list
    :returns: list
    """
    storage_manager = StorageManager(repo)
    schema = get_schema(repo, content_type).to_json()
    model_class = deserialize(schema, module_name=schema['namespace'])
    return [dict(model_obj)
            for model_obj in storage_manager.iterate(model_class)]


def format_content_type_object(repo, content_type, uuid):
    """
    Return a content object from a repository for a given content_type
    and uuid

    :param Repo repo:
        The git repository.
    :param str content_type:
        The content type to list
    :returns: dict
    """
    try:
        storage_manager = StorageManager(repo)
        schema = get_schema(repo, content_type).to_json()
        model_class = deserialize(schema, module_name=schema['namespace'])
        return dict(storage_manager.get(model_class, uuid))
    except GitCommandError:
        raise NotFound('Object does not exist.')


def format_repo_status(repo):
    """
    Return a dictionary representing the repository status

    It returns ``None`` for things we do not support or are not
    relevant.

    :param str repo_name:
        The name of the repository.
    :returns: dict

    """
    commit = repo.commit()
    return {
        'name': os.path.basename(repo.working_dir),
        'commit': commit.hexsha,
        'timestamp': datetime.fromtimestamp(
            commit.committed_date).isoformat(),
    }


def save_content_type_object(repo, schema, uuid, data):
    """
    Save an object as a certain content type
    """
    storage_manager = StorageManager(repo)
    model_class = deserialize(schema,
                              module_name=schema['namespace'])
    model = model_class(data)
    commit = storage_manager.store(model, 'Updated via PUT request.')
    return commit, model


def delete_content_type_object(repo, content_type, uuid):
    """
    Delete an object of a certain content type
    """
    storage_manager = StorageManager(repo)
    schema = get_schema(repo, content_type).to_json()
    model_class = deserialize(schema, module_name=schema['namespace'])
    model = storage_manager.get(model_class, uuid)
    commit = storage_manager.delete(model, 'Deleted via DELETE request.')
    return commit, model


def get_config(request):  # pragma: no cover
    """
    Get the configuration for a request.

    :param Request request:
        The HTTP request
    """
    return request.registry.settings


def get_schema_names(repo):
    schema_files = glob.glob(
        os.path.join(repo.working_dir, '_schemas', '*.avsc'))
    names = []
    for schema_file in schema_files:
        names.append(os.path.basename(os.path.splitext(schema_file)[0]))
    return names


def add_model_item_to_pull_dict(storage_manager, path, pull_dict):
    if path.endswith(".json"):
        model = storage_manager.load(path)
        pull_dict[model.__module__ + "." +
                  model.__class__.__name__].append(dict(model))
        return True
    return False


def get_repository_diff(repo, commit_id):
    try:
        old_commit = repo.commit(commit_id)
        diff = old_commit.diff(repo.head)

        return {
            "name": os.path.basename(repo.working_dir),
            "previous-index": commit_id,
            "current-index": repo.commit().hexsha,
            "diff": list(format_diffindex(diff))
        }
    except (GitCommandError, BadName):
        raise NotFound("The git index does not exist")


def pull_repository_files(repo, commit_id):
    changed_files = {}
    for name in get_schema_names(repo):
        changed_files[name] = []

    try:
        old_commit = repo.commit(commit_id)
        diff = old_commit.diff(repo.head)

        sm = StorageManager(repo)
        for diff_added in diff.iter_change_type('A'):
            add_model_item_to_pull_dict(
                sm, diff_added.b_blob.path, changed_files)

        for diff_modified in diff.iter_change_type('M'):
            add_model_item_to_pull_dict(
                sm, diff_modified.b_blob.path, changed_files)

        json_diff = []
        for diff_added in diff.iter_change_type('R'):
            json_diff.append(format_diff_R(diff_added))

        for diff_removed in diff.iter_change_type('D'):
            json_diff.append(format_diff_D(diff_removed))

        changed_files["other"] = json_diff
        changed_files["commit"] = repo.head.commit.hexsha
        return changed_files

    except (GitCommandError, BadName):
        raise NotFound("The git index does not exist")


def clone_repository(repo):
    get_schema_names(repo)
    files = {}
    for name in get_schema_names(repo):
        files[name] = format_content_type(repo, name)

    files['commit'] = repo.head.commit.hexsha
    return files



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as f:
    README = f.read()

with open(os.path.join(here, 'requirements.txt')) as f:
    requires = filter(None, f.readlines())

with open(os.path.join(here, 'VERSION')) as f:
    version = f.read().strip()

setup(name='unicore.distribute',
      version=version,
      description='Distribution tools for Universal Core',
      long_description=README,
      classifiers=[
          "Programming Language :: Python",
          "Framework :: Pyramid",
          "Topic :: Internet :: WWW/HTTP",
          "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
      ],
      author='Praekelt Foundation',
      author_email='dev@praekelt.com',
      url='http://github.com/universalcore/unicore.distribute',
      license='BSD',
      keywords='universal, core',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      namespace_packages=['unicore'],
      install_requires=requires,
      tests_require=requires,
      entry_points={
          'paste.app_factory': ['main = unicore.distribute.api:main'],
          'console_scripts': [
              'unicore.distribute = unicore.distribute.scripts:main',
          ]
      })



import glob
import json
import textwrap
from StringIO import StringIO
from unittest import TestCase

import os
from elasticgit.tests.base import TestPerson, ModelBaseTest
from elasticgit.commands.avro import serialize
from pyramid import testing
from unicore.distribute.utils import (
    UCConfigParser, get_repositories, get_repository, format_repo,
    format_content_type, format_content_type_object, list_schemas, get_schema,
    get_repository_diff, pull_repository_files, add_model_item_to_pull_dict,
    get_schema_names, clone_repository)
from git import Repo


class TestUCConfigParser(TestCase):
    def setUp(self):
        sio = StringIO(textwrap.dedent("""
            [foo]
            list =
                1
                2
                3

            dict =
                a = 1
                b = 2
                c = 3
            """))
        self.cp = UCConfigParser()
        self.cp.readfp(sio)

    def test_get_list(self):
        self.assertEqual(self.cp.get_list('foo', 'list'), ['1', '2', '3'])

    def test_get_dict(self):
        self.assertEqual(self.cp.get_dict('foo', 'dict'), {
            'a': '1',
            'b': '2',
            'c': '3',
        })


class TestRepositoryUtils(ModelBaseTest):
    maxDiff = None
    initial_commit = ""

    def setUp(self):
        self.workspace = self.mk_workspace()

    def test_get_repositories(self):
        [repo] = get_repositories(self.WORKING_DIR)
        self.assertTrue(isinstance(repo, Repo))
        self.assertEqual(repo.working_dir, self.workspace.working_dir)

    def test_get_repository(self):
        repo = get_repository(self.workspace.working_dir)
        self.assertTrue(isinstance(repo, Repo))
        self.assertEqual(repo.working_dir, self.workspace.working_dir)

    def test_format_repo(self):
        formatted = format_repo(self.workspace.repo)
        self.assertEqual(
            set(formatted.keys()),
            set(['name',
                 'commit',
                 'timestamp',
                 'author',
                 'schemas',
                 'branch']))
        last_commit = self.workspace.repo.commit()
        self.assertEqual(formatted['commit'], last_commit.hexsha)
        self.assertEqual(
            formatted['author'], '%s <%s>' % (last_commit.author.name,
                                              last_commit.author.email))

    def test_format_content_type(self):
        p = TestPerson({'name': 'Foo', 'age': 1})
        schema_string = serialize(TestPerson)
        schema = json.loads(schema_string)
        self.workspace.save(p, 'Saving a person.')
        self.workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')
        [model_obj] = format_content_type(
            self.workspace.repo, '%(namespace)s.%(name)s' % schema)
        self.assertEqual(TestPerson(model_obj), p)

    def test_format_content_type_object(self):
        p = TestPerson({'name': 'Foo', 'age': 1})
        schema_string = serialize(TestPerson)
        schema = json.loads(schema_string)
        self.workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')
        self.workspace.save(p, 'Saving a person.')
        model_obj = format_content_type_object(
            self.workspace.repo, '%(namespace)s.%(name)s' % schema,
            p.uuid)
        self.assertEqual(TestPerson(model_obj), p)

    def test_list_schemas(self):
        schema_string = serialize(TestPerson)
        schema = json.loads(schema_string)
        self.workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')
        schemas = list_schemas(self.workspace.repo)
        found_schema = schemas['%(namespace)s.%(name)s' % schema]
        self.assertEqual(found_schema['namespace'], schema['namespace'])
        self.assertEqual(found_schema['name'], schema['name'])

    def test_get_schema(self):
        schema_string = serialize(TestPerson)
        schema = json.loads(schema_string)
        self.workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')
        found_schema = get_schema(
            self.workspace.repo,
            '%(namespace)s.%(name)s' % schema).to_json()
        self.assertEqual(found_schema['namespace'], schema['namespace'])
        self.assertEqual(found_schema['name'], schema['name'])


class TestRepositoryUtils2(ModelBaseTest):
    """
    Separate class because initialising repo on setup above breaks other tests
    """

    initial_commit = ""
    schema_names = []

    def setUp(self):
        self.workspace = self.mk_workspace()
        self.add_schema(self.workspace, TestPerson)
        self.config = testing.setUp(settings={
            'repo.storage_path': self.WORKING_DIR,
        })
        person1 = TestPerson({'age': 12, 'name': 'Foo'})
        person2 = TestPerson({'age': 34, 'name': 'Bar'})
        self.create_commit("initial commit")
        self.workspace.save(person1, "saving person 1")
        self.workspace.save(person2, "saving person 2")
        self.create_commit("second commit")
        self.schema_names = set(self.create_schema_names())

    def add_schema(self, workspace, model_class):
        schema_string = serialize(model_class)
        schema = json.loads(schema_string)
        workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')

    def create_schema_names(self):
        schema_files = glob.glob(os.path.join(self.workspace.repo.working_dir,
                                              '_schemas', '*.avsc'))
        names = []
        for schema_file in schema_files:
            names.append(os.path.basename(os.path.splitext(schema_file)[0]))
        return names

    def create_commit(self, message):
        repo = Repo(self.workspace.working_dir)
        repo.commit(repo.index.commit(message))
        return repo.commit().hexsha

    def test_get_schema_names(self):
        schema = get_schema_names(self.workspace.repo)
        self.assertGreater(len(schema), 0)
        self.assertEqual(set(schema),
                         {TestPerson.__module__ + '.' + TestPerson.__name__})

    def test_add_model_item_to_pull_dict(self):
        person = TestPerson({'age': 22, 'name': 'testing'})
        self.workspace.save(person, 'saving person testing')
        path = self.workspace.sm.git_name(person)
        response = {TestPerson.__module__ + '.' + TestPerson.__name__: []}
        added = add_model_item_to_pull_dict(self.workspace.sm, path, response)
        self.assertTrue(added)

    def test_get_repository_diff(self):
        diff = get_repository_diff(self.workspace.repo, self.initial_commit)
        self.assertEqual(set(diff.keys()),
                         {'current-index', 'previous-index', 'name', 'diff'})
        self.assertEqual(diff['current-index'],
                         self.workspace.repo.head.commit.hexsha)
        self.assertEqual(diff['previous-index'],
                         self.initial_commit)
        self.assertEqual(diff['name'],
                         os.path.basename(self.workspace.repo.working_dir))

    def test_pull_repository_files(self):
        pull = pull_repository_files(self.workspace.repo, self.initial_commit)
        self.assertEqual(set(pull.keys()),
                         {'commit', 'other'} | self.schema_names)
        self.assertEqual(pull['commit'],
                         self.workspace.repo.head.commit.hexsha)

    def test_clone_repository(self):
        clone = clone_repository(self.workspace.repo)
        self.assertEqual(set(clone), {'commit'} | self.schema_names)
        self.assertEqual(clone['commit'],
                         self.workspace.repo.head.commit.hexsha)



from __future__ import absolute_import

import json
import os

from cornice.errors import Errors

from elasticgit import EG
from elasticgit.storage import StorageManager
from elasticgit.tests.base import TestPerson, ModelBaseTest
from elasticgit.commands.avro import serialize
from elasticgit.utils import fqcn

from git.exc import GitCommandError

from pyramid import testing
from pyramid.exceptions import NotFound

from mock import patch
import avro

from unicore.distribute.api.repos import (
    RepositoryResource, ContentTypeResource)
from unicore.distribute.utils import (
    format_repo, format_content_type, format_content_type_object)


class TestRepositoryResource(ModelBaseTest):

    def setUp(self):
        self.workspace = self.mk_workspace()
        self.add_schema(self.workspace, TestPerson)
        self.config = testing.setUp(settings={
            'repo.storage_path': self.WORKING_DIR,
        })

    def add_schema(self, workspace, model_class):
        schema_string = serialize(model_class)
        schema = json.loads(schema_string)
        workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')

    def create_upstream_for(self, workspace, create_remote=True,
                            remote_name='origin',
                            suffix='upstream'):
        upstream_workspace = self.mk_workspace(
            name='%s_%s' % (self.id().lower(), suffix),
            index_prefix='%s_%s' % (self.workspace.index_prefix,
                                    suffix))
        if create_remote:
            workspace.repo.create_remote(
                remote_name, upstream_workspace.working_dir)
        return upstream_workspace

    def test_collection_get(self):
        request = testing.DummyRequest({})
        resource = RepositoryResource(request)
        [repo_json] = resource.collection_get()
        self.assertEqual(repo_json, format_repo(self.workspace.repo))

    def test_collection_post_success(self):
        # NOTE: cloning to a different directory called `remote` because
        #       the API is trying to clone into the same folder as the
        #       tests: self.WORKING_DIR.
        #
        # FIXME: This is too error prone & tricky to reason about
        api_repo_name = '%s_remote' % (self.id(),)
        self.remote_workspace = self.mk_workspace(
            working_dir=os.path.join(self.WORKING_DIR, 'remote'),
            name=api_repo_name)
        request = testing.DummyRequest({})
        request.validated = {
            'repo_url': self.remote_workspace.working_dir,
        }
        # Cleanup the repo created by the API on tear down
        self.addCleanup(
            lambda: EG.workspace(
                os.path.join(
                    self.WORKING_DIR, api_repo_name)).destroy())
        request.route_url = lambda route, name: (
            '/repos/%s.json' % (api_repo_name,))
        request.errors = Errors()
        resource = RepositoryResource(request)
        resource.collection_post()
        self.assertEqual(
            request.response.headers['Location'],
            '/repos/%s.json' % (api_repo_name,))
        self.assertEqual(request.response.status_code, 301)

    @patch.object(EG, 'clone_repo')
    def test_collection_post_error(self, mock_method):
        mock_method.side_effect = GitCommandError(
            'git clone', 'Boom!', stderr='mocked response')
        request = testing.DummyRequest({})
        request.validated = {
            'repo_url': 'git://example.org/bar.git',
        }
        request.errors = Errors()
        resource = RepositoryResource(request)
        resource.collection_post()
        [error] = request.errors
        self.assertEqual(error['location'], 'body')
        self.assertEqual(error['name'], 'repo_url')
        self.assertEqual(error['description'], 'mocked response')

    def test_get(self):
        request = testing.DummyRequest({})
        repo_name = os.path.basename(self.workspace.working_dir)
        request.matchdict = {
            'name': repo_name,
        }
        resource = RepositoryResource(request)
        repo_json = resource.get()
        self.assertEqual(repo_json, format_repo(self.workspace.repo))

    def test_get_404(self):
        request = testing.DummyRequest({})
        request.matchdict = {
            'name': 'does-not-exist',
        }
        resource = RepositoryResource(request)
        self.assertRaises(NotFound, resource.get)

    @patch.object(StorageManager, 'pull')
    def test_post(self, mock_pull):
        request = testing.DummyRequest({})
        request.route_url = lambda route, name: 'foo'
        repo_name = os.path.basename(self.workspace.working_dir)
        request.matchdict = {
            'name': repo_name,
        }
        request.params = {
            'branch': 'foo',
            'remote': 'bar',
        }
        resource = RepositoryResource(request)

        with patch.object(request.registry, 'notify') as mocked_notify:
            resource.post()
            mock_pull.assert_called_with(branch_name='foo',
                                         remote_name='bar')
            (call,) = mocked_notify.call_args_list
            (args, kwargs) = call
            (event,) = args
            self.assertEqual(event.event_type, 'repo.push')
            self.assertEqual(event.payload, {
                'repo': repo_name,
                'url': 'foo',
            })

    def test_pull_additions(self):
        upstream_workspace = self.create_upstream_for(self.workspace)
        self.add_schema(upstream_workspace, TestPerson)
        person1 = TestPerson({'age': 1, 'name': 'person1'})
        upstream_workspace.save(person1, 'Adding person1.')

        repo_name = os.path.basename(self.workspace.working_dir)
        request = testing.DummyRequest({})
        request.route_url = lambda route, name: 'foo'
        request.matchdict = {
            'name': repo_name,
        }

        resource = RepositoryResource(request)
        [diff] = resource.post()
        self.assertEqual(diff, {
            'type': 'A',
            'path': upstream_workspace.sm.git_name(person1),
        })

    def test_pull_removals(self):
        upstream_workspace = self.create_upstream_for(self.workspace)
        self.add_schema(upstream_workspace, TestPerson)
        person1 = TestPerson({'age': 1, 'name': 'person1'})
        upstream_workspace.save(person1, 'Adding person1.')
        self.workspace.pull()
        upstream_workspace.delete(person1, 'Removing person1.')

        repo_name = os.path.basename(self.workspace.working_dir)
        request = testing.DummyRequest({})
        request.route_url = lambda route, name: 'foo'
        request.matchdict = {
            'name': repo_name,
        }

        resource = RepositoryResource(request)
        [diff] = resource.post()
        self.assertEqual(diff, {
            'type': 'D',
            'path': upstream_workspace.sm.git_name(person1),
        })

    def test_pull_renames(self):
        upstream_workspace = self.create_upstream_for(self.workspace)
        self.add_schema(upstream_workspace, TestPerson)
        upstream_workspace.sm.store_data(
            'README.rst', 'the readme', 'Writing the readme.')
        self.workspace.pull()
        # do the rename
        upstream_workspace.repo.index.move(['README.rst', 'README.txt'])
        upstream_workspace.repo.index.commit('Renaming rst to txt.')

        repo_name = os.path.basename(self.workspace.working_dir)
        request = testing.DummyRequest({})
        request.route_url = lambda route, name: 'foo'
        request.matchdict = {
            'name': repo_name,
        }

        resource = RepositoryResource(request)
        [diff] = resource.post()
        self.assertEqual(diff, {
            'type': 'R',
            'rename_from': 'README.rst',
            'rename_to': 'README.txt',
        })

    def test_pull_modified(self):
        upstream_workspace = self.create_upstream_for(self.workspace)
        self.add_schema(upstream_workspace, TestPerson)
        person1 = TestPerson({'age': 1, 'name': 'person1'})
        upstream_workspace.save(person1, 'Adding person1.')
        self.workspace.pull()
        updated_person1 = person1.update({'age': 2})
        upstream_workspace.save(updated_person1, 'Updating person1.')

        repo_name = os.path.basename(self.workspace.working_dir)
        request = testing.DummyRequest({})
        request.route_url = lambda route, name: 'foo'
        request.matchdict = {
            'name': repo_name,
        }

        resource = RepositoryResource(request)
        [diff] = resource.post()
        self.assertEqual(diff, {
            'type': 'M',
            'path': upstream_workspace.sm.git_name(person1),
        })


class TestContentTypeResource(ModelBaseTest):

    def setUp(self):
        self.workspace = self.mk_workspace()
        schema_string = serialize(TestPerson)
        schema = json.loads(schema_string)
        self.workspace.sm.store_data(
            os.path.join(
                '_schemas',
                '%(namespace)s.%(name)s.avsc' % schema),
            schema_string, 'Writing the schema.')
        self.person = TestPerson({'name': 'Foo', 'age': 1})
        self.workspace.save(self.person, 'Saving a person.')
        self.config = testing.setUp(settings={
            'repo.storage_path': self.WORKING_DIR,
        })

    def test_collection(self):
        request = testing.DummyRequest({})
        request.matchdict = {
            'name': os.path.basename(self.workspace.working_dir),
            'content_type': fqcn(TestPerson),
        }
        resource = ContentTypeResource(request)
        content_type_json = resource.collection_get()
        self.assertEqual(
            content_type_json, format_content_type(self.workspace.repo,
                                                   fqcn(TestPerson)))

    def test_get(self):
        request = testing.DummyRequest({})
        request.matchdict = {
            'name': os.path.basename(self.workspace.working_dir),
            'content_type': fqcn(TestPerson),
            'uuid': self.person.uuid,
        }
        resource = ContentTypeResource(request)
        object_json = resource.get()
        self.assertEqual(
            object_json, format_content_type_object(self.workspace.repo,
                                                    fqcn(TestPerson),
                                                    self.person.uuid))

    def test_get_404(self):
        request = testing.DummyRequest({})
        request.matchdict = {
            'name': os.path.basename(self.workspace.working_dir),
            'content_type': fqcn(TestPerson),
            'uuid': 'does not exist',
        }
        resource = ContentTypeResource(request)
        self.assertRaises(NotFound, resource.get)

    def test_put(self):
        request = testing.DummyRequest()
        request.schema = avro.schema.parse(serialize(TestPerson)).to_json()
        request.schema_data = dict(self.person)
        request.matchdict = {
            'name': os.path.basename(self.workspace.working_dir),
            'content_type': fqcn(TestPerson),
            'uuid': self.person.uuid,
        }
        request.registry = self.config.registry
        resource = ContentTypeResource(request)
        object_data = resource.put()
        self.assertEqual(TestPerson(object_data), self.person)

    def test_delete(self):
        request = testing.DummyRequest()
        request.matchdict = {
            'name': os.path.basename(self.workspace.working_dir),
            'content_type': fqcn(TestPerson),
            'uuid': self.person.uuid,
        }
        resource = ContentTypeResource(request)
        object_data = resource.delete()
        self.assertEqual(TestPerson(object_data), self.person)

        request = testing.DummyRequest({})
        request.matchdict = {
            'name': os.path.basename(self.workspace.working_dir),
            'content_type': fqcn(TestPerson),
            'uuid': self.person.uuid,
        }
        resource = ContentTypeResource(request)
        self.assertRaises(NotFound, resource.get)



import os
from urlparse import urlparse

from cornice.resource import resource, view

from elasticgit.storage import StorageManager

from unicore.distribute.api.validators import (
    validate_schema, CreateRepoColanderSchema)
from unicore.webhooks.events import WebhookEvent
from unicore.distribute.utils import (
    get_config, get_repositories, get_repository, format_repo,
    format_content_type, format_content_type_object,
    save_content_type_object, delete_content_type_object,
    format_diffindex)

from git.exc import GitCommandError
from elasticgit import EG


@resource(collection_path='/repos.json', path='/repos/{name}.json')
class RepositoryResource(object):

    def __init__(self, request):
        self.request = request
        self.config = get_config(request)

    def collection_get(self):
        storage_path = self.config.get('repo.storage_path')
        return [format_repo(repo) for repo in get_repositories(storage_path)]

    @view(schema=CreateRepoColanderSchema)
    def collection_post(self):
        storage_path = self.config.get('repo.storage_path')
        repo_url = self.request.validated['repo_url']
        repo_url_info = urlparse(repo_url)
        repo_name_dot_git = os.path.basename(repo_url_info.path)
        repo_name = repo_name_dot_git.partition('.git')[0]
        try:
            EG.clone_repo(repo_url, os.path.join(storage_path, repo_name))
            self.request.response.headers['Location'] = self.request.route_url(
                'repositoryresource', name=repo_name)
            self.request.response.status = 301
            return ''
        except (GitCommandError,), e:
            self.request.errors.status = 400
            self.request.errors.add(
                'body', 'repo_url', e.stderr)

    @view(renderer='json')
    def get(self):
        name = self.request.matchdict['name']
        storage_path = self.config.get('repo.storage_path')
        return format_repo(get_repository(os.path.join(storage_path, name)))

    @view(renderer='json')
    def post(self):
        name = self.request.matchdict['name']
        branch_name = self.request.params.get('branch', 'master')
        remote_name = self.request.params.get('remote')
        storage_path = self.config.get('repo.storage_path')
        repo = get_repository(os.path.join(storage_path, name))
        storage_manager = StorageManager(repo)
        changes = storage_manager.pull(branch_name=branch_name,
                                       remote_name=remote_name)
        # Fire an event
        self.request.registry.notify(
            WebhookEvent(
                owner=self.request.authenticated_userid,
                event_type='repo.push',
                payload={
                    'repo': name,
                    'url': self.request.route_url('repositoryresource',
                                                  name=name)
                }))
        return format_diffindex(changes)


@resource(collection_path='/repos/{name}/{content_type}.json',
          path='/repos/{name}/{content_type}/{uuid}.json')
class ContentTypeResource(object):

    def __init__(self, request):
        self.request = request
        self.config = get_config(request)

    def collection_get(self):
        name = self.request.matchdict['name']
        content_type = self.request.matchdict['content_type']
        storage_path = self.config.get('repo.storage_path')
        return format_content_type(
            get_repository(os.path.join(storage_path, name)),
            content_type)

    @view(renderer='json', validators=validate_schema)
    def put(self):
        name = self.request.matchdict['name']
        uuid = self.request.matchdict['uuid']
        storage_path = self.config.get('repo.storage_path')
        commit, model = save_content_type_object(
            get_repository(os.path.join(storage_path, name)),
            self.request.schema, uuid, self.request.schema_data)
        return dict(model)

    @view(renderer='json')
    def get(self):
        name = self.request.matchdict['name']
        content_type = self.request.matchdict['content_type']
        uuid = self.request.matchdict['uuid']
        storage_path = self.config.get('repo.storage_path')
        return format_content_type_object(
            get_repository(os.path.join(storage_path, name)),
            content_type, uuid)

    @view(renderer='json')
    def delete(self):
        name = self.request.matchdict['name']
        content_type = self.request.matchdict['content_type']
        uuid = self.request.matchdict['uuid']
        storage_path = self.config.get('repo.storage_path')
        commit, model = delete_content_type_object(
            get_repository(os.path.join(storage_path, name)),
            content_type, uuid)
        return dict(model)



from pyramid.config import Configurator

from unicore.distribute.api import proxy


def main(global_config, **settings):
    config = Configurator(settings=settings)
    config.include('unicore.distribute.api')
    return config.make_wsgi_app()


def includeme(config):
    config.include('cornice')
    config.scan('.repo_status')
    config.scan('.repos')

    settings = config.registry.settings
    proxy_enabled = settings.get('proxy.enabled', 'false').lower()
    proxy_path = settings.get('proxy.path', 'esapi')
    proxy_upstream = settings.get('proxy.upstream', 'http://localhost:9200/')

    if proxy_enabled == 'true':  # pragma: no cover
        config.add_route('esapi', '/%s/{parts:.*}' % (proxy_path,))
        config.add_view(proxy.Proxy(proxy_upstream), route_name='esapi')
